LOG_LEVEL = 'STOOBLY_AGENT_LOG_LEVEL'
AGENT_ACTIVE_MODE = 'STOOBLY_AGENT_ACTIVE_MODE'
AGENT_ENABLED = 'STOOBLY_AGENT_ENABLED'
AGENT_IS_HEADLESS = 'STOOBLY_AGENT_IS_HEADLESS'
AGENT_INCLUDE_PATTERNS = 'STOOBLY_AGENT_INCLUDE_PATTERNS'
AGENT_EXCLUDE_PATTERNS = 'STOOBLY_AGENT_EXCLUDE_PATTERNS'
AGENT_POLICY = 'STOOBLY_AGENT_POLICY'
AGENT_SERVICE_URL = 'STOOBLY_AGENT_SERVICE_URL'
AGENT_PROJECT_KEY = 'STOOBLY_AGENT_PROJECT_KEY'
AGENT_PROXY_URL = 'STOOBLY_AGENT_PROXY_URL'
AGENT_SCENARIO_KEY = 'STOOBLY_AGENT_SCENARIO_KEY'
AGENT_URL = 'STOOBLY_AGENT_URL'
API_URL = 'STOOBLY_API_URL'
API_KEY = 'STOOBLY_API_KEY'
